import React, { useState } from "react";
import styled from "styled-components";
import ClickToCopy from "../../util/ClickToCopy";
import Button from "../../util/Button";
import Loading from "../../util/Loading";
import PlayerNameInput from "../../util/PlayerNameInput";
import LobbyPlayerList from "./LobbyPlayerList";
import { joinGame } from "../../../domain/GGSHService";
import DelayedButton from "../../util/DelayedButton";
import useObservable from "../../../hooks/useObservable";
import { playerNameStore, gameIdStore, gameJoinedStore, gameStateStore } from "../../../domain/store";
import { IN_ROUND } from "../../../domain/GameState";
import HeaderAndFooter from "../../util/HeaderAndFooter";

const InfoContainer = styled.div`
  flex: 3;
  align-self: center;

  display: flex;
  flex-flow: column nowrap;
  justify-content: center;
  align-items: center;
`

const Label = styled.span`
  padding: 0.2em;
`

const FormButton = styled(Button)`
  padding: 1em;
  margin: 0.2em;
  margin-top: 0.3em;
`

const StyledDelayButton = styled(DelayedButton)`
  padding: 1em;
  margin: 0.2em;
  margin-top: 0.3em;
`

const PageContainer = styled.div`
  flex: 1;
  display: flex;
  flex-flow: row nowrap;
  justify-content: space-between;
  align-items: flex-start;
`

const getUrl = gameId => {
  const u = new URL(window.location.href);
  u.searchParams.append("join", gameId);
  return u.href;
}

const LobbyInfo = () => {
  const gameId = useObservable(gameIdStore);
  const playerName = useObservable(playerNameStore);

  return (
    <InfoContainer>
      <Label>Playing as {playerName}</Label>
      <Label>Use this link to invite other players:</Label>
      <Label><ClickToCopy text={getUrl(gameId)} /></Label>
      <StyledDelayButton 
        onEnd={() => gameStateStore.set(IN_ROUND)} 
        countDownFormatter={rem => `Click to cancel, ${rem}s...`}
      >
        Start Game
      </StyledDelayButton>
    </InfoContainer>
  )
};

const JoinForm = () => {
  const gameId = useObservable(gameIdStore);
  const playerName = useObservable(playerNameStore);
  const [loading, setLoading] = useState(false);
  const [failed, setFailed] = useState(false);

  if (loading) {
    return <InfoContainer><Loading/></InfoContainer>
  }

  const onJoinGame = async () => {
    // TODO would like to support re-joining a game you left
    setLoading(true);
    try {
      await joinGame();
    } catch (err) {
      // failed to join the game
      setFailed(true);
      setLoading(false);
      return;
    }
    setFailed(false);
    playerNameStore.set(playerName);
    gameJoinedStore.set(true);
  };
  const cannotJoinGame = !playerName || playerName.length === 0;

  return (
    <InfoContainer>
      {failed && <Label>Failed to join the game! Maybe try a different name?</Label>}
      <Label>Joining {gameId}...</Label>
      <PlayerNameInput/>
      <FormButton onClick={onJoinGame} disabled={cannotJoinGame}>Join Game</FormButton>
    </InfoContainer>
  );
}

const PreRound = ({ onStart }) => {
  const joined = useObservable(gameJoinedStore);

  return (
    <HeaderAndFooter>
      <PageContainer>
        {
          joined
            ? <LobbyInfo onStart={onStart}/>
            : <JoinForm />
        }
        <LobbyPlayerList/>
      </PageContainer>
    </HeaderAndFooter>
  )
};

export default PreRound;
